/**
 * @file
 * @brief
 *
 * @author  Anton Kozlov
 * @date    07.08.2014
 */

#include <util/log.h>

#include <drivers/net/stm32cube_eth.h>

/*****************************************************************************
 Ethernet MSP Routines
 *****************************************************************************/
/**
 * @brief  Initializes the ETH MSP.
 * @param  heth: ETH handle
 * @retval None
 */
void HAL_ETH_MspInit(ETH_HandleTypeDef *heth) {
	GPIO_InitTypeDef GPIO_InitStructure;

	/* Enable GPIOs clocks */
	__HAL_RCC_GPIOA_CLK_ENABLE()
	;
	__HAL_RCC_GPIOC_CLK_ENABLE()
	;
	__HAL_RCC_GPIOG_CLK_ENABLE();

	/* Ethernet pins configuration *****************************************/
	/*
	 RMII_REF_CLK ----------------------> PA1
	 RMII_MDIO -------------------------> PA2
	 RMII_MDC --------------------------> PC1
	 RMII_MII_CRS_DV -------------------> PA7
	 RMII_MII_RXD0 ---------------------> PC4
	 RMII_MII_RXD1 ---------------------> PC5
	 RMII_MII_RXER ---------------------> PG2
	 RMII_MII_TX_EN --------------------> PG11
	 RMII_MII_TXD0 ---------------------> PG13
	 RMII_MII_TXD1 ---------------------> PG14
	 */

	/* Configure PA1, PA2 and PA7 */
	GPIO_InitStructure.Speed = GPIO_SPEED_HIGH;
	GPIO_InitStructure.Mode = GPIO_MODE_AF_PP;
	GPIO_InitStructure.Pull = GPIO_NOPULL;
	GPIO_InitStructure.Alternate = GPIO_AF11_ETH;
	GPIO_InitStructure.Pin = GPIO_PIN_1 | GPIO_PIN_2 | GPIO_PIN_7;
	HAL_GPIO_Init(GPIOA, &GPIO_InitStructure);

	/* Configure PC1, PC4 and PC5 */
	GPIO_InitStructure.Pin = GPIO_PIN_1 | GPIO_PIN_4 | GPIO_PIN_5;
	HAL_GPIO_Init(GPIOC, &GPIO_InitStructure);

	/* Configure PG2, PG11, PG13 and PG14 */
	GPIO_InitStructure.Pin =
			GPIO_PIN_2 | GPIO_PIN_11 | GPIO_PIN_13 | GPIO_PIN_14;
	HAL_GPIO_Init(GPIOG, &GPIO_InitStructure);

	/* Enable ETHERNET clock  */
	__HAL_RCC_ETH_CLK_ENABLE();
}
